// Add new constant to support read-only mode for test purposes
const EDIT_MODE_TP_RO  = 0x02;

// Command function: Add new test purpose as requirement's child or another testpurpose's sibling
function addTestPurpose(uuid) {
	if ((readOnlyMode & EDIT_MODE_TP_RO) != 0)
		return;
	// Get the wrapping block of the target requirement (if it is a requirement)
	// or of the target test purpose's parent requirement (if it is a test purpose)
	var req_elem = $(this).closest('blockquote.indent');
	// Create the child test purpose if there are no children requirements
	if (!req_elem.hasClass('collapsable'))
		addNode(extractUUID(req_elem[0].id), null, 'TestPurpose');
}

$(function () {
	// Register new hotkey for adding test purposes
	hotkeyManager.registerHotkeys({
		'Ins': { handlerGlobal: addTestPurpose, handlerCKEditor: null }
	});

	// Implement Save actions for test purpose specific fields
	saveAction['tpid'] = function (uuid, new_data) {
		changeNodeId(uuid, new_data);
	};
	saveAction['tpdesc'] = function (uuid, new_data) {
		// Split to Description and Expected Results
		var desc;
		var exp_res;
		var pos = new_data.indexOf('-- Expected results:');
		if (pos >= 0) {
			desc = new_data.substr(0, pos).replace(/<br\s*\/>\s*$/, '');
			exp_res = new_data.substr(pos + 20).replace(/^\s*<br\s*\/>/, '');
			// Collect all tags that remain opened at the "Expected results" delimiter
			var tag_stack = [];
			pos = 0;
			while ((pos = desc.indexOf('<', pos)) != -1) {
				// Tag started...
				var pos_end = desc.indexOf('>', pos);
				if (pos_end == -1)
					break;   // Corrupted XHTML, stop parsing
				if (desc[pos + 1] == '/') {
					// This is a closing tag - remove from the stack
					var tag_name = desc.substring(pos + 2, pos_end);
					var last_tag = tag_stack.pop();
					if (tag_name != last_tag.replace(/ .*$/, '')) {
						tag_stack.push(last_tag);
					}
				}
				else {
					if (desc[pos_end - 1] != '/') {
						// This is an opening and not self-closing tag - push to the stack
						tag_stack.push(desc.substring(pos + 1, pos_end));
					}
				}
				pos = pos_end;
			}
			// Close all opened tags in the end of first part (Description)
			// and duplicate them at the beginning of the second part (Expected results)
			var exp_res_prefix = '';
			var last_tag;
			while (last_tag = tag_stack.pop()) {
				desc += '</' + last_tag.replace(/ .*$/, '') + '>';
				exp_res_prefix = '<' + last_tag + '>' + exp_res_prefix;
			}
			exp_res = exp_res_prefix + exp_res;
		}
		else {
			desc = new_data;
			exp_res = '';
		}
		changeNodeAttr(uuid, '_description', desc);
		changeNodeAttr(uuid, '_expectedResults', exp_res);
	};

	// Add read-only class binding
	readOnlyClasses[EDIT_MODE_TP_RO] = 'testpurpose';

	// Append command to Requirement nodes for creating test purpose children
	node_commands['Requirement']['addtp'] = {
		menu: true,
		name: 'Add new test purpose',
		image: 'add-tp.png',
		imageAlt: '[+]',
		readOnlyMode: EDIT_MODE_TP_RO,
		handler: addTestPurpose
	};
	// Add drop-down menu to test purpose nodes
	node_commands['TestPurpose'] = {
		'deltp': {
			menu: true,
			name: 'Delete test purpose',
			image: 'delete-node.png',
			imageAlt: '[&minus;]',
			readOnlyMode: EDIT_MODE_TP_RO,
			handler: deleteNodeConfirm
		},
		'addtp': {
			menu: true,
			name: 'Add new test purpose',
			image: 'add-tp.png',
			imageAlt: '[+]',
			readOnlyMode: EDIT_MODE_TP_RO,
			handler: addTestPurpose
		},
		'addcmt': {
			menu: true,
			name: 'Add new comment',
			image: 'add-cmt.png',
			imageAlt: '[+]',
			readOnlyMode: 0,
			handler: addComment
		},
		'chstatus': {
			menu: false,
			name: 'Change status',
			readOnlyMode: 0,
			handler: changeStatus
		}
	};

	// Fill in list of statuses
	node_statuses['TestPurpose'] = {
		'in process': {
			icon: 'tp-status-incomplete.png',
			className: 'status-normal'
		},
		'complete': {
			icon: 'tp-status-complete.png',
			className: 'status-complete'
		},
		'verified': {
			icon: 'tp-status-verified.png',
			className: 'status-verified'
		}
	};

	// Insert TestPurpose node type into the order list at the last but one position (just before Requirement)
	childrenNodesOrder.splice(-1, 0, 'TestPurpose');

	// Add test purpose node constructors
	constructNodeWrapperProcessor['TestPurpose'] = function (uuid) {
		return $('<div class="testpurposes"><div class="testpurpose-title">Test purposes:</div></div>');
	};

	constructNodeProcessor['TestPurpose'] = function (nodeData, uuid) {
		// Fill in default values for missing attributes
		if (!nodeData.attributes._description || !nodeData.attributes._description.value)
			nodeData.attributes._description = { 'value': '&nbsp;' };
		if (!nodeData.attributes._status || !nodeData.attributes._status.value)
			nodeData.attributes._status = { 'value': 'in process' };
		// Combine description and expected results together
		var expectedResults = '';
		if (nodeData.attributes._expectedResults && nodeData.attributes._expectedResults.value)
			expectedResults = '<br />-- Expected results:<br />' + nodeData.attributes._expectedResults.value;
		// Construct HTML block
		var editable_prefix = (((readOnlyMode & EDIT_MODE_TP_RO) == 0) ? '' : 'non');
		var newNode = $(
			'<div class="tpwrapper nodewrapper" id="block-' + uuid + '">' +
			  '<div style="height: 1px; width: 100%; border-top: solid 1px gray;"></div>' +
			  '<div class="testpurpose supported-node" id="node-' + uuid + '">' +
			    '<table cellpadding="0" cellspacing="0" border="0" width="100%" class="tpid_wrap">' +
			      '<tr>' +
			        '<td>' +
			          '<img src="images/' + node_statuses['TestPurpose'][nodeData.attributes._status.value].icon + '" alt="' + nodeData.attributes._status.value + '" title="' + nodeData.attributes._status.value + '" id="status-' + uuid + '" class="imgstatus" onclick="javascript:execNodeCommand(\'TestPurpose\', \'' + uuid + '\', \'chstatus\');" />' +
			        '</td>' +
			        '<td width="100%">' +
			          '<div class="tpid nodetitle ' + editable_prefix + 'renamable ' + node_statuses['TestPurpose'][nodeData.attributes._status.value].className + '" id="tpid-' + uuid + '">' +
			            nodeData.id +
			          '</div>' +
			        '</td>' +
			        '<td>' +
			          '<div class="tpctrls nodectrls">' +
			            '<div style="text-align: right;">' +
			              '<img src="images/menu.png" alt="[v]" />' +
			            '</div>' +
			            '<div class="menu" style="display: none;">' +
			              constructMenu('TestPurpose', uuid) +
			            '</div>' +
			          '</div>' +
			        '</td>' +
			      '</tr>' +
			    '</table>' +
			    '<div class="tpdesc nodedesc ' + editable_prefix + 'editable" id="tpdesc-' + uuid + '">' +
			      nodeData.attributes._description.value + expectedResults +
			    '</div>' +
			  '</div>' +
			'</div>'
		);

		// Enumerate all children objects and recursively construct the rest of the tree
		var childrenNodes = {};
		for (var i in nodeData.children) {
			// Construct the next child subtree (for requirements) or table (for test purposes)
			var chldNode = constructTree(nodeData.children[i]);
			var chldType = chldNode.type;
			if (!childrenNodes[chldType]) {
				var wrapperConstructor = constructNodeWrapperProcessor[chldType];
				if (wrapperConstructor)
					childrenNodes[chldType] = wrapperConstructor(uuid);
				else
					childrenNodes[chldType] = $('<div />');
			}
			childrenNodes[chldType].append(chldNode);
		}
		// Now append children nodes blocks to the current requirement node
		for (var i = 0; i < childrenNodesOrder.length; ++i) {
			var chldType = childrenNodesOrder[i];
			if (childrenNodes[chldType]) {
				newNode.append(childrenNodes[chldType]);
			}
		}

		return newNode;
	};

	// Override Requirement constructor to remove "Add Test Purpose" command from non-leaves
	var oldRequirementProcessor = constructNodeProcessor['Requirement'];
	constructNodeProcessor['Requirement'] = function (nodeData, uuid) {
		var newNode = oldRequirementProcessor(nodeData, uuid);
		if (newNode.hasClass('collapsable'))
			$(newNode.find('.menu-item-addtp')[0]).remove();
		return newNode;
	};

	// Add class-to-type mapping for new node type
	nodeClassToType['testpurpose'] = 'TestPurpose';

	// Allow paste comments into test purposes
	pasteAllowedMatrix['Comment'].push('TestPurpose');
	// Allow paste test purposes into requirements
	pasteAllowedMatrix['TestPurpose'] = [ 'Requirement' ];

	// Add name of the status attribute for the new node type
	statusAttr['TestPurpose'] = '_status';

	// Customize CKEditor style for editing test purpose ID
	CKEditorStyles['tpid'] = 'body { margin: 1px; font-weight: bold; }';

	// Add specific dragging init handler
	draggingInitHandler['tpid'] = function (dragging_uuid, req_elems) {
		var req_uuid = extractUUID($('#node-' + dragging_uuid).closest('blockquote')[0].id);
		var underlying_req_idx = req_elems.index($('#node-' + req_uuid));
		if (underlying_req_idx < 0) {
			underlying_req_idx = null;
		}
		// Disallow dragging onto the parent requirement, and onto all non-leaves
		$('#node-' + req_uuid + ', .collapsable > .requirement').addClass('undroppable');
		// Set the disabled look for parent requirement and all non-leaves
		$('#block-' + req_uuid + ', .collapsable > .requirement, .collapsable > .testpurposes').addClass('disabled');
		return underlying_req_idx;
	};
});
