/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.editors.browser;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.BrowserFunction;
import org.eclipse.swt.browser.LocationEvent;
import org.eclipse.swt.browser.LocationListener;
import org.eclipse.swt.browser.ProgressEvent;
import org.eclipse.swt.browser.ProgressListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.nodetypes.Location;

public class ReqDocBrowser extends Composite {
    enum BrowserState {
        NEW, LOADED, READY
    }

    private Browser webBrowser;
    private BrowserState state = BrowserState.NEW;
    private static final boolean DEBUG = true;
    private boolean commandRuning = false;
    private List<IJSCommand> jsCommands = new ArrayList<IJSCommand>();

    public boolean isReady() {
        return state.equals(BrowserState.READY);
    }

    public ReqDocBrowser(Composite parent, int style) {
        super(parent, style);

        this.setLayout(new FillLayout());
        webBrowser = new Browser(this, RequalityPlugin.BROWSER_TYPE);
        new BrowserFunction(webBrowser, "reportDocumentLoaded") {
            @Override
            public Object function(Object[] arguments) {
                state = BrowserState.READY;
                runCommands();
                return true;
            }
        };
        webBrowser.setVisible(false);

        webBrowser.addProgressListener(new ProgressListener() {
            @Override
            public void completed(ProgressEvent event) {
                if (state.equals(BrowserState.NEW)) {
                    state = BrowserState.LOADED;
                    injectScripts();
                    webBrowser.setVisible(true);
                }
            }

            @Override
            public void changed(ProgressEvent event) {
            }
        });
        webBrowser.addLocationListener(new LocationListener() {
            @Override
            public void changing(LocationEvent event) {
                if (!state.equals(BrowserState.NEW)) {
                    event.doit = false;
                }
            }

            @Override
            public void changed(LocationEvent event) {
            }
        });
    }

    private void executeJS(final String jsCode) {
        scheduleJSCommand(new IJSCommand() {
            @Override
            public void onError(Exception e) {
                e.printStackTrace();
            }

            @Override
            public String getJS() {
                return jsCode;
            }

            @Override
            public void onComplite(Object result, ReqDocBrowser docBrowser) {
            }

            @Override
            public boolean isEvaluate() {
                return false;
            }
        });
    }

    protected void runCommands() {
        getDisplay().syncExec(new Runnable() {
            @Override
            public void run() {
                while (jsCommands.size() > 0) {
                    try {
                        List<IJSCommand> cmd2proc = new ArrayList<IJSCommand>(
                                jsCommands);
                        jsCommands.clear();
                        for (IJSCommand jsCode : cmd2proc) {
                            runJSCommand(jsCode);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
                jsCommands.clear();
            }
        });
    }
    private void runJSCommand(IJSCommand jsCommand) {
        commandRuning = true;
        String js = jsCommand.getJS();
        if (DEBUG) {
            if(jsCommand.isEvaluate()) {
                js = "try{" + js + "}catch(e){alert(e);alert(\""
                    + js.replace('"', '`') + "\");}";
            } else {
                js = "try{res = " + js + "}catch(e){alert(e);alert(\""
                        + js.replace('"', '`') + "\");}";
            }
        }
        try {
            Object result;
            if (jsCommand.isEvaluate()) {
                result = webBrowser.evaluate(js);
            } else {
                result = webBrowser.execute(js);
            }
            jsCommand.onComplite(result, this);
            return;
        } catch (Exception e) {
            jsCommand.onError(e);
        } finally {
            commandRuning = false;
        }
    }

    protected void scheduleJSCommand(IJSCommand jsCommand) {
        // add command to queue
        jsCommands.add(jsCommand);
        if (!state.equals(BrowserState.READY) || commandRuning) {
            return;
        }
        runCommands();
    }

    public String serializeDocument() {
        return (String) webBrowser.evaluate("return serializeDocument();");
    }

    public String serializeDocumentRaw() {
        return (String) webBrowser.evaluate("return serializeDocumentRaw();");
    }

    protected void injectScripts() {
        // TODO check if <head> element is found
        String jsPath = Activator.getDefault().getJSToolsPath();
        String script = "var h = document.getElementsByTagName('head')[0];"
                + "var s1 = document.createElement('script');"
                + "s1.setAttribute('id','jquery_js');"
                + "s1.setAttribute('type','text/javascript');"
                + "s1.setAttribute('src','" + jsPath + "jquery/jquery.js');"
                + "var s2 = document.createElement('script');"
                + "s2.setAttribute('id','edit_js');"
                + "s2.setAttribute('type','text/javascript');"
                + "s2.setAttribute('src','" + jsPath + "edit.js');"
                + "var s3 = document.createElement('link');"
                + "s3.setAttribute('id','edit_css');"
                + "s3.setAttribute('rel','stylesheet');"
                + "s3.setAttribute('type','text/css');"
                + "s3.setAttribute('href','" + jsPath + "edit.css');"
                + "var s4 = document.createElement('script');"
                + "s4.setAttribute('id','blackbird_js');"
                + "s4.setAttribute('type','text/javascript');"
                + "s4.setAttribute('src','" + jsPath + "blackbird/blackbird.js');"
                + "var s5 = document.createElement('link');"
                + "s5.setAttribute('id','blackbird_css');"
                + "s5.setAttribute('rel','stylesheet');"
                + "s5.setAttribute('type','text/css');"
                + "s5.setAttribute('href','" + jsPath + "blackbird/blackbird.css');"
                + "h.appendChild(s4);" + "h.appendChild(s5);"
                + "h.appendChild(s1);" + "h.appendChild(s2);"
                + "h.appendChild(s3);"
                ;
        webBrowser.execute(script);
    }

    public void selectNode() {
        executeJS("selectNode();");
    }

    public void showBrowserInfo() {
        executeJS("alert(getBrowserInfo());");
    }

    protected boolean toBoolean(Object obj) {
        return obj != null ? (Boolean) obj : false;
    }

    public boolean isSelectable() {
        return toBoolean(webBrowser.evaluate("return isSelectable();"));
    }

    public boolean isRemovable() {
        return toBoolean(webBrowser.evaluate("return isRemovable();"));
    }

    /**
     * @return [ String text, String action ]
     */
    public Object[] createLocation(String id, String reqId) {
        Object[] textAndAction = (Object[]) webBrowser
                .evaluate("return createLocation('" + id + "','" + reqId
                        + "');");
        return textAndAction;
    }

    public String deleteLocation(String locationId) {
        return (String) webBrowser.evaluate("return removeLocation('"
                + locationId + "');");
    }

    public String getSelectionText() {
        String res = (String) webBrowser.evaluate("return getSelectionText();");
        if (res == null)
            return "";
        StringTokenizer st = new StringTokenizer(res, "\n ");
        res = "";
        while (st.hasMoreTokens()) {
            if (!res.isEmpty()) {
                res += " ";
            }
            res += st.nextToken();
        }
        return res;
    }

    public String getSelectedLocationId() {
        return (String) webBrowser
                .evaluate("return getSelectedLocationUUID();");
    }

    public void activateLocation(String locationId) {
        executeJS("activateLocation('" + locationId + "',true);");
    }

    public void activateSelectedLocation() {
        executeJS("activateSelectedLocation();");
    }

    public boolean findText(String aString, boolean aCaseSensitive,
            boolean aBackwards, boolean aWrapAround) {
        return (Boolean) webBrowser.evaluate("return findText('" + aString
                + "'," + aCaseSensitive + "," + aBackwards + "," + aWrapAround
                + ");");
    }

    private Location location;

    public void setUrl(String url) {
        webBrowser.setUrl(url);
    }

    public Browser getWebBrowser() {
        return webBrowser;
    }

    @Override
    public boolean setFocus() {
        return webBrowser.setFocus();
    }

    /**
     * @param targetRequirement
     *            - Target requirement or null
     */
    public void setTargetLocation(Location location) {
        if (location != null) {
            this.location = location;
            activateLocation(location.getId());
        } else {
            System.out
                    .println("ReqDocBrowser.setTargetLocation(Location location): argumen is null");
            // TODO: why location is null?
        }
    }

    public Location getTargetLocation() {
        return location;
    }

    public void updateLocations() {
        executeJS("removeAllLinks();");
        executeJS("addLocationLinks();");
    }

    public Control getMozilla() {
        return webBrowser;
    }
}
