/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.handlers.document;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.DocFolder;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.tools.DeepFirstTreeWalker;

public class DeleteDocumentHandler extends AbstractHandler {
    private static final String DELETE_DOCUMENT = "Delete Document";
    private static final String DELETE_FOLDER = "Delete Folder";

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        TreeNode node = RequalityCNF.getSelectedNode();
        TreeNode parent = node.getParent();
        if (node instanceof DocFolder) {
            if (!MessageDialog
                    .openConfirm(PlatformUI.getWorkbench()
                            .getActiveWorkbenchWindow().getShell(),
                            DELETE_FOLDER,
                            "Delete selected Document Folder and all its nested documents and subfolders?")) {
                return null;
            }
            if (folderHasDocumentWithLocation((DocFolder) node)) {
                if (!MessageDialog
                        .openConfirm(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(),
                                DELETE_FOLDER,
                                "Folder contains document with marked requirements, continue?")) {
                    return null;
                }
            }
            node.getTreeDB().startTransaction("Delete Document Folder");
            recDelNode(node);
            node.getTreeDB().commit();
        }
        if (node instanceof Document) {
            if (!MessageDialog.openConfirm(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), DELETE_DOCUMENT,
                    "Delete selected Document?")) {
                return null;
            }
            if (((Document) node).sizeChildren() > 0) {
                if (!MessageDialog.openConfirm(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), DELETE_FOLDER,
                        "Document contains marked requirements, continue?")) {
                    return null;
                }
            }
            // TODO: document delete undo
//            node.getTreeDB().startTransaction("Delete Document");
            IUndoContext context = PlatformUI.getWorkbench().getOperationSupport().getUndoContext();
            PlatformUI.getWorkbench().getOperationSupport().getOperationHistory().dispose(context, true, true, true);
//
            deleteDocument((Document) node);
//            node.getTreeDB().commit();
        }
        if (parent != null)
            Activator.getDefault().setActiveNode(parent, parent.getType());
        return null;
    }

    @Override
    public boolean isEnabled() {
        TreeNode node = RequalityCNF.getSelectedNode();
        if (node == null) {
            return false;
        }
        if (node.getQualifiedId().equals(Document.getTypeRootQId())) {
            return false;
        }
        if (!(node.getType().equals("Document") || node.getType().equals(
                "DocFolder"))) {
            return false;
        }
        return true;
    }

    //start transaction and commit, please!
    private void recDelNode(TreeNode node) {
        if (node instanceof Document) {
            deleteDocument((Document) node);
        }

        if (node instanceof DocFolder) {
            while (node.sizeChildren() > 0) {
                recDelNode(node.getChild(0));
            }
            node.delete();
        }
    }

    //start transaction and commit, please!
    private void deleteDocument(Document doc) {
        removeLocationsFromRequirementd(doc);
        doc.delete();
    }

    private boolean folderHasDocumentWithLocation(DocFolder folder) {
        if (folder.sizeChildren() == 0) {
            return false;
        }

        for (int i = 0; i < folder.sizeChildren(); i++) {
            TreeNode node = folder.getChild(i);
            if (node instanceof DocFolder) {
                if (folderHasDocumentWithLocation((DocFolder) node)) {
                    return true;
                }
            }
            if (node instanceof Document) {
                Document doc = (Document) node;
                if (doc.sizeLocations() > 0) {
                    return true;
                }
            }
        }
        return false;
    }

    //start transaction and commit, please!
    private void removeLocationsFromRequirementd(Document doc) {
        TreeDB db = doc.getTreeDB();
        Set<String> locQIds = new HashSet<String>();
        for (int i = 0; i < doc.sizeChildren(); i++) {
            locQIds.add(doc.getChild(i).getQualifiedId());
        }

        for (TreeNode node : new DeepFirstTreeWalker(db.getNode(Requirement
                .getTypeRootQId()))) {

            if (!node.getType().equals("Requirement")) {
                continue;
            }

            ArrayList<Location> removeLocations = new ArrayList<Location>();
            Requirement req = (Requirement) node;

            for (Location loc : req.getLocations()) {
                if (locQIds.contains(loc.getQualifiedId())) {
                    removeLocations.add(loc);
                }
            }

            for (Location loc : removeLocations) {
                req.removeLocation(loc.getQualifiedId());
            }
            req.saveAttributes();
        }
    }
}
